<?php

/**
 * @copyright  https://github.com/UsabilityDynamics/zoom-api-php-client/blob/master/LICENSE
 */
namespace Zoom\Interfaces\Oauth;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Psr7\Response;

class Request {

    /**
     * @var
     */
    protected $accountId;

    /**
     * @var
     */
    protected $clientId;

    /**
     * @var
     */
    protected $clientSecret;
    /**
     * @var
     */
    protected $oAuthToken;

    protected $config;

    /**
     * @var string
     */
    public $apiPoint = 'https://api.zoom.us/v2/';

    /**
     * @var string
     */
    public $oAuthTokenUrl = 'https://zoom.us/oauth/token';

    /**
     * Request constructor.
     * @param $accountId
     * @param $clientId
     * @param $clientSecret
     */
    public function __construct($accountId, $clientId, $clientSecret) {
        require_once JPATH_ADMINISTRATOR.'/components/com_rsform/helpers/config.php';

        $this->accountId = $accountId;
        $this->clientId = $clientId;
        $this->clientSecret = $clientSecret;
		
		$this->config = \RSFormProConfig::getInstance();
		 
		$this->client = new Client();
		
        $this->oAuthToken = $this->config->get('zoom.oauthtoken', '');
		
        // check the validity of the token
        if (strlen($this->oAuthToken))
        {
            $this->oAuthToken = \json_decode($this->oAuthToken);
			$this->oAuthToken = (array) $this->oAuthToken;
            $this->checkOAuthToken($this->oAuthToken);
        }
        else
        {
            $this->getAndStoreToken();
        }
    }
	
	protected function getAndStoreToken() {
		$this->oAuthToken = $this->requestOAuthToken();
			
		if ($this->oAuthToken['code'] !== 200)
		{
			throw new \Exception('Could not retrieve OAuth Token!'.print_r($this->oAuthToken, true));
		}
		
		$this->storeToken($this->oAuthToken);
	}

    protected function checkOAuthToken($oAuthToken)
    {
		$last_check = $this->config->get('zoom.oauthtokendate');
		$expires = strtotime($last_check) + (int)$oAuthToken['expires_in'];
		
		$jdate = new \JDate;
		$current = $jdate->toSql();
		$current = strtotime($current);
		
		if ($current >= $expires)
		{
			 $this->getAndStoreToken();
		}
    }

    protected function storeToken($token)
    {
        $token = \json_encode($token);
		$this->config->set('zoom.oauthtoken', $token);
		
		// hold the date
		$jdate = new \JDate;
		$this->config->set('zoom.oauthtokendate', $jdate->toSql());
    }

    /**
     * Headers
     *
     * @return array
     * @throws \Exception
     */
    protected function headers(): array {
        if (is_null($this->oAuthToken))
        {
            throw new \Exception('No OAuth Token provided!');
        }
        return [
            'Authorization' => 'Bearer ' .  $this->oAuthToken['access_token'],
            'Content-Type' => 'application/json',
            'Accept' => 'application/json',
            'Host' => 'api.zoom.us',
        ];
    }

    protected function oAuthHeaders(): array {
        return [
            'Authorization' => 'Basic '. base64_encode($this->clientId . ':' . $this->clientSecret),
            'Host' => 'zoom.us',
        ];
    }

    /**
     * Generate oAuth Token
     *
     * @return array
     */
    protected function requestOAuthToken()
    {
        try {
            $response = $this->client->request('POST', $this->oAuthTokenUrl . '?grant_type=account_credentials&account_id='.$this->accountId, [
                'headers' => $this->oAuthHeaders(),
            ]);

            return $this->result($response);

        } catch (ClientException $e) {

            return (array)json_decode($e->getResponse()->getBody()->getContents());
        }
    }


    /**
     * Get
     *
     * @param $method
     * @param array $fields
     * @return array|mixed
     */
    protected function get($method, $fields = []) {
        try {
            $response = $this->client->request('GET', $this->apiPoint . $method, [
                'query' => $fields,
                'headers' => $this->headers(),
            ]);

            return $this->result($response);

        } catch (ClientException $e) {

            return (array)json_decode($e->getResponse()->getBody()->getContents());
        }
    }

    /**
     * Post
     *
     * @param $method
     * @param $fields
     * @return array|mixed
     */
    protected function post($method, $fields, $query = []) {
        $body = \json_encode($fields, JSON_PRETTY_PRINT);

        try {
            $response = $this->client->request('POST', $this->apiPoint . $method,
                ['query' => $query, 'body' => $body, 'headers' => $this->headers()]);

            return $this->result($response);

        } catch (ClientException $e) {

            return (array)json_decode($e->getResponse()->getBody()->getContents());
        }
    }

    /**
     * Patch
     *
     * @param $method
     * @param $fields
     * @return array|mixed
     */
    protected function patch($method, $fields) {
        $body = \json_encode($fields, JSON_PRETTY_PRINT);

        try {
            $response = $this->client->request('PATCH', $this->apiPoint . $method,
                ['body' => $body, 'headers' => $this->headers()]);

            return $this->result($response);

        } catch (ClientException $e) {

            return (array)json_decode($e->getResponse()->getBody()->getContents());
        }
    }

    /**
     * Put
     *
     * @param $method
     * @param $fields
     * @return array|mixed
     */
    protected function put($method, $fields) {
        $body = \json_encode($fields, JSON_PRETTY_PRINT);

        try {
            $response = $this->client->request('PUT', $this->apiPoint . $method,
                ['body' => $body, 'headers' => $this->headers()]);

            return $this->result($response);

        } catch (ClientException $e) {

            return (array)json_decode($e->getResponse()->getBody()->getContents());
        }
    }

    /**
     * Delete
     *
     * @param $method
     * @param $fields
     * @return array|mixed
     */
    protected function delete($method, $fields = []) {
        $body = \json_encode($fields, JSON_PRETTY_PRINT);

        try {
            $response = $this->client->request('DELETE', $this->apiPoint . $method,
                ['body' => $body, 'headers' => $this->headers()]);

            return $this->result($response);

        } catch (ClientException $e) {

            return (array)json_decode($e->getResponse()->getBody()->getContents());
        }
    }

    /**
     * Result
     *
     * @param Response $response
     * @return mixed
     */
    protected function result(Response $response) {
        $result = json_decode((string)$response->getBody(), true);

        $result['code'] = $response->getStatusCode();

        return $result;
    }
}